@@ -69,7 +69,7 @@ struct _NodesX
 
  node_t   *idata;               /*+ The extended node IDs (sorted by ID). +*/
 
- node_t   *gdata;               /*+ The final node indexes (sorted geographically). +*/
+ index_t  *gdata;               /*+ The final node indexes (sorted geographically). +*/
 
  uint8_t  *super;               /*+ A bit-mask marker for super nodes (same order as sorted nodes). +*/
 
@@ -103,11 +103,6 @@ void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx);
 
 /* Macros and inline functions */
 
-#define ClearBit(xx,yy)    (xx)[(yy)/8]&=~(1<<((yy)%8))
-#define SetBit(xx,yy)      (xx)[(yy)/8]|= (1<<((yy)%8))
-#define IsBitSet(xx,yy)   ((xx)[(yy)/8]&  (1<<((yy)%8)))
-
-
 #if !SLIM
 
 #define LookupNodeX(nodesx,index,position)      &(nodesx)->data[index]
