@@ -573,7 +573,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
             }
          }
 
-       if(!option_quiet && !(results->number%10000))
+       if(!option_quiet && !(results->number%1000))
           printf_middle("Routing: Super-Nodes checked = %d",results->number);
 
       endloop:
@@ -729,7 +729,6 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
     index_t node1,seg1,seg1r;
     Segment *segment;
     index_t turnrelation=NO_RELATION;
-    int     routes_out=0;
 
     node1=result1->node;
     seg1=result1->segment;
@@ -826,8 +825,6 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,Relations *r
              goto endloop;
          }
 
-       routes_out++;
-
        if(option_quickest==0)
           segment_score=(score_t)DISTANCE(segment->distance)/segment_pref;
        else
