@@ -398,7 +398,7 @@ void SortTurnRelationList(RelationsX* relationsx)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Sort the turn restriction relations into via node order (then by from and to segments).
+  Sort the turn restriction relations into via index order (then by from and to segments).
 
   int sort_by_via Returns the comparison of the via, from and to fields.
 
@@ -409,8 +409,8 @@ void SortTurnRelationList(RelationsX* relationsx)
 
 static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b)
 {
- node_t a_id=a->via;
- node_t b_id=b->via;
+ index_t a_id=a->via;
+ index_t b_id=b->via;
 
  if(a_id<b_id)
     return(-1);
@@ -418,8 +418,8 @@ static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b)
     return(1);
  else
    {
-    node_t a_id=a->from;
-    node_t b_id=b->from;
+    index_t a_id=a->from;
+    index_t b_id=b->from;
 
     if(a_id<b_id)
        return(-1);
@@ -427,8 +427,8 @@ static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b)
        return(1);
     else
       {
-       node_t a_id=a->to;
-       node_t b_id=b->to;
+       index_t a_id=a->to;
+       index_t b_id=b->to;
 
        if(a_id<b_id)
           return(-1);
@@ -452,7 +452,7 @@ static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b)
 void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 {
  RouteRelX *unmatched=NULL,*lastunmatched=NULL;
- int nunmatched=0,lastnunmatched=0,iteration=0;
+ int nunmatched=0,lastnunmatched=0,iteration=1;
 
  if(waysx->number==0)
     return;
@@ -474,13 +474,13 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
  do
    {
     int ways=0,relations=0;
-    int i;
+    index_t i;
 
     SeekFile(relationsx->rfd,0);
 
     /* Print the start message */
 
-    printf_first("Processing Route Relations: Iteration=%d Relations=0 Modified Ways=0",iteration);
+    printf_first("Processing Route Relations (%d): Relations=0 Modified Ways=0",iteration);
 
     for(i=0;i<relationsx->rnumber;i++)
       {
@@ -497,7 +497,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
        /* Decide what type of route it is */
 
-       if(iteration==0)
+       if(iteration==1)
          {
           relations++;
           routes=relationx.routes;
@@ -570,8 +570,8 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
          }
        while(relationid);
 
-       if(!((i+1)%10000))
-          printf_middle("Processing Route Relations: Iteration=%d Relations=%d Modified Ways=%d",iteration,relations,ways);
+       if(!((i+1)%1000))
+          printf_middle("Processing Route Relations (%d): Relations=%d Modified Ways=%d",iteration,relations,ways);
       }
 
     if(lastunmatched)
@@ -585,9 +585,9 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
     /* Print the final message */
 
-    printf_last("Processed Route Relations: Iteration=%d Relations=%d Modified Ways=%d",iteration,relations,ways);
+    printf_last("Processed Route Relations (%d): Relations=%d Modified Ways=%d",iteration,relations,ways);
    }
- while(lastnunmatched && ++iteration<5);
+ while(lastnunmatched && iteration++<5);
 
  if(lastunmatched)
     free(lastunmatched);
@@ -618,11 +618,12 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
 void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
 {
- int i,trfd;
+ int trfd;
+ index_t i,deleted=0;
 
  /* Print the start message */
 
- printf_first("Processing Turn Restriction Relations (1): Turn Relations=0");
+ printf_first("Processing Turn Relations (1): Relations=0");
 
  /* Re-open the file read-only and a new file writeable */
 
@@ -644,10 +645,13 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
     relationx.from=IndexWayX(waysx,relationx.from);
     relationx.to  =IndexWayX(waysx,relationx.to);
 
-    WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
+    if(relationx.via==NO_NODE || relationx.from==NO_WAY || relationx.to==NO_WAY)
+       deleted++;
+    else
+       WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
 
-    if(!((i+1)%10000))
-       printf_middle("Processing Turn Restriction Relations (1): Turn Relations=%d",i+1);
+    if(!((i+1)%1000))
+       printf_middle("Processing Turn Relations (1): Relations=%d Deleted=%d",i+1-deleted,deleted);
    }
 
  /* Close the files */
@@ -657,7 +661,9 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
 
  /* Print the final message */
 
- printf_last("Processed Turn Restriction Relations (1): Turn Relations=%d",relationsx->trnumber);
+ printf_last("Processed Turn Relations (1): Relations=%d Deleted=%d",relationsx->trnumber-deleted,deleted);
+
+ relationsx->trnumber-=deleted;
 }
 
 
@@ -677,14 +683,14 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 {
  TurnRestrictRelX relationx;
  int trfd;
- int total=0,deleted=0;
+ index_t total=0,deleted=0;
 
  if(nodesx->number==0 || segmentsx->number==0)
     return;
 
  /* Print the start message */
 
- printf_first("Processing Turn Restriction Relations (2): Turn Relations=0");
+ printf_first("Processing Turn Relations (2): Relations=0");
 
  /* Map into memory / open the files */
 
@@ -716,7 +722,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
        relationx.restrict==TurnRestrict_no_u_turn ||
        relationx.restrict==TurnRestrict_no_straight_on)
       {
-       node_t node_from=NO_NODE,node_to=NO_NODE;
+       index_t node_from=NO_NODE,node_to=NO_NODE;
 
        /* Find the segments that join the node 'via' */
 
@@ -767,12 +773,12 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
        total++;
 
-       if(!(total%10000))
-          printf_middle("Processing Turn Restriction Relations (2): Turn Relations=%d Deleted=%d Added=%d",total,deleted,total-relationsx->trnumber+deleted);
+       if(!(total%1000))
+          printf_middle("Processing Turn Relations (2): Relations=%d Deleted=%d Added=%d",total,deleted,total-relationsx->trnumber+deleted);
       }
     else
       {
-       node_t node_from=NO_NODE,node_to[16];
+       index_t node_from=NO_NODE,node_to[MAX_SEG_PER_NODE];
        int nnodes_to=0,i;
 
        /* Find the segments that join the node 'via' */
@@ -794,11 +800,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
           if(segmentx->way!=relationx.to)
             {
-             if(nnodes_to==16)
-               {
-                deleted++;
-                goto endloop;
-               }
+             assert(nnodes_to<MAX_SEG_PER_NODE); /* Only a limited amount of information stored. */
 
              /* Don't bother with restrictions banning going the wrong way down a one-way road */
 
@@ -829,8 +831,8 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
           total++;
 
-          if(!(total%10000))
-             printf_middle("Processing Turn Restriction Relations (2): Turn Relations=%d Deleted=%d Added=%d",total,deleted,total-relationsx->trnumber+deleted);
+          if(!(total%1000))
+             printf_middle("Processing Turn Relations (2): Relations=%d Deleted=%d Added=%d",total,deleted,total-relationsx->trnumber+deleted);
          }
       }
 
@@ -873,7 +875,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
  /* Print the final message */
 
- printf_last("Processed Turn Restriction Relations (2): Turn Relations=%d Deleted=%d Added=%d",total,deleted,total-relationsx->trnumber+deleted);
+ printf_last("Processed Turn Relations (2): Relations=%d Deleted=%d Added=%d",total,deleted,total-relationsx->trnumber+deleted);
 
  relationsx->trnumber=total;
 }
@@ -892,11 +894,11 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx)
 {
  int trfd;
- int i;
+ index_t i;
 
  /* Print the start message */
 
- printf_first("Updating Turn Restriction Relations: Turn Relations=0");
+ printf_first("Updating Turn Relations: Relations=0");
 
  /* Map into memory / open the files */
 
@@ -946,8 +948,8 @@ void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmen
 
     WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
 
-    if(!(relationsx->trnumber%10000))
-       printf_middle("Updating Turn Restriction Relations: Turn Relations=%d",relationsx->trnumber);
+    if(!(relationsx->trnumber%1000))
+       printf_middle("Updating Turn Relations: Relations=%d",relationsx->trnumber);
    }
 
  /* Close the files */
@@ -965,7 +967,7 @@ void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmen
 
  /* Print the final message */
 
- printf_last("Updated Turn Restriction Relations: Turn Relations=%d",relationsx->trnumber);
+ printf_last("Updated Turn Relations: Relations=%d",relationsx->trnumber);
 }
 
 
@@ -1011,7 +1013,7 @@ void SaveRelationList(RelationsX* relationsx,const char *filename)
 
     WriteFile(fd,&relation,sizeof(TurnRelation));
 
-    if(!((i+1)%10000))
+    if(!((i+1)%1000))
        printf_middle("Writing Relations: Turn Relations=%d",i+1);
    }
 
