@@ -285,14 +285,14 @@ SegmentX *FirstSegmentX(SegmentsX *segmentsx,index_t nodeindex,int position)
 
   SegmentX *segmentx The current segment.
 
-  index_t node The node index.
+  index_t nodeindex The node index.
 
   int position A flag to pass through.
   ++++++++++++++++++++++++++++++++++++++*/
 
-SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t node,int position)
+SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t nodeindex,int position)
 {
- if(segmentx->node1==node)
+ if(segmentx->node1==nodeindex)
    {
 #if SLIM
     index_t index=IndexSegmentX(segmentsx,segmentx);
@@ -301,13 +301,13 @@ SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t node,int
     if(index>=segmentsx->number)
        return(NULL);
     segmentx=LookupSegmentX(segmentsx,index,position);
-    if(segmentx->node1!=node)
+    if(segmentx->node1!=nodeindex)
        return(NULL);
     else
        return(segmentx);
 #else
     segmentx++;
-    if(IndexSegmentX(segmentsx,segmentx)>=segmentsx->number || segmentx->node1!=node)
+    if(IndexSegmentX(segmentsx,segmentx)>=segmentsx->number || segmentx->node1!=nodeindex)
        return(NULL);
     else
        return(segmentx);
@@ -333,14 +333,14 @@ SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t node,int
 
 void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 {
- int duplicate=0,loop=0,missing=0,good=0,total=0;
+ index_t duplicate=0,loop=0,nonode=0,good=0,total=0;
  SegmentX segmentx;
  int fd;
- node_t prevnode1=NO_NODE,prevnode2=NO_NODE;
+ node_t prevnode1=NO_NODE_ID,prevnode2=NO_NODE_ID;
 
  /* Print the start message */
 
- printf_first("Checking Segments: Segments=0 Duplicate=0 Loop=0 Missing-Node=0");
+ printf_first("Checking Segments: Segments=0 Duplicate=0 Loop=0 No-Node=0");
 
  /* Allocate the array of node flags */
 
@@ -368,7 +368,7 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
     else if(segmentx.node1==segmentx.node2)
        loop++;
     else if(index1==NO_NODE || index2==NO_NODE)
-       missing++;
+       nonode++;
     else
       {
        WriteFile(fd,&segmentx,sizeof(SegmentX));
@@ -385,7 +385,7 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
     total++;
 
     if(!(total%10000))
-       printf_middle("Checking Segments: Segments=%d Duplicate=%d Loop=%d Missing-Node=%d",total,duplicate,loop,missing);
+       printf_middle("Checking Segments: Segments=%d Duplicate=%d Loop=%d No-Node=%d",total,duplicate,loop,nonode);
    }
 
  segmentsx->number=good;
@@ -397,7 +397,7 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 
  /* Print the final message */
 
- printf_last("Checked Segments: Segments=%d Duplicate=%d Loop=%d Missing-Node=%d",total,duplicate,loop,missing);
+ printf_last("Checked Segments: Segments=%d Duplicate=%d Loop=%d No-Node=%d",total,duplicate,loop,nonode);
 }
 
 
@@ -507,12 +507,12 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
 void DeduplicateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 {
- int duplicate=0,good=0;
+ index_t duplicate=0,good=0;
  index_t index=0;
  int fd,nprev=0;
- node_t prevnode1=NO_NODE,prevnode2=NO_NODE;
- SegmentX prevsegx[16],segmentx;
- Way prevway[16];
+ index_t prevnode1=NO_NODE,prevnode2=NO_NODE;
+ SegmentX prevsegx[MAX_SEG_PER_NODE],segmentx;
+ Way prevway[MAX_SEG_PER_NODE];
 
  /* Print the start message */
 
@@ -567,7 +567,7 @@ void DeduplicateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
          }
        else
          {
-          assert(nprev<(sizeof(prevsegx)/sizeof(prevsegx[0]))); /* Only a limited amount of history stored. */
+          assert(nprev<MAX_SEG_PER_NODE); /* Only a limited amount of information stored. */
 
           prevsegx[nprev]=segmentx;
           prevway[nprev] =wayx->way;
@@ -630,8 +630,7 @@ void DeduplicateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
 void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
 {
- index_t index;
- int i;
+ index_t index,i;
 
  if(segmentsx->number==0)
     return;
@@ -792,7 +791,7 @@ void SaveSegmentList(SegmentsX *segmentsx,const char *filename)
  index_t i;
  int fd;
  SegmentsFile segmentsfile={0};
- int super_number=0,normal_number=0;
+ index_t super_number=0,normal_number=0;
 
  /* Print the start message */
 
