@@ -20,6 +20,7 @@
  ***************************************/
 
 
+#include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -53,7 +54,7 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
 void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 {
  index_t i;
- int nnodes=0;
+ index_t nnodes=0;
 
  if(nodesx->number==0 || segmentsx->number==0 || waysx->number==0)
     return;
@@ -84,14 +85,12 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
        NodeX *nodex=LookupNodeX(nodesx,i,1);
 
        if(nodex->flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
-         {
           issuper=1;
-         }
        else
          {
           int count=0,j;
-          Way segmentway[16];
-          int segmentweight[16];
+          Way segmentway[MAX_SEG_PER_NODE];
+          int segmentweight[MAX_SEG_PER_NODE];
           SegmentX *segmentx=FirstSegmentX(segmentsx,i,1);
 
           while(segmentx)
@@ -101,7 +100,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
              /* Segments that are loops count twice */
 
-             assert(count<(sizeof(segmentway)/sizeof(segmentway[0]))); /* Only a limited amount of history stored. */
+             assert(count<MAX_SEG_PER_NODE); /* Only a limited amount of information stored. */
 
              if(segmentx->node1==segmentx->node2)
                 segmentweight[count]=2;
@@ -196,7 +195,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 {
  index_t i;
  SegmentsX *supersegmentsx;
- int sn=0,ss=0;
+ index_t sn=0,ss=0;
 
  supersegmentsx=NewSegmentList(0);
 
@@ -225,7 +224,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
       {
        SegmentX *segmentx;
        int count=0,match;
-       Way prevway[16];
+       Way prevway[MAX_SEG_PER_NODE];
 
        segmentx=FirstSegmentX(segmentsx,i,1);
 
@@ -249,7 +248,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
                   }
             }
 
-          assert(count<(sizeof(prevway)/sizeof(prevway[0]))); /* Only a limited amount of history stored. */
+          assert(count<MAX_SEG_PER_NODE); /* Only a limited amount of history stored. */
 
           prevway[count++]=wayx->way;
 
@@ -319,7 +318,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
 {
  index_t i,j;
- int m=0,a=0;
+ index_t merged=0,added=0;
  SegmentsX *mergedsegmentsx;
 
  mergedsegmentsx=NewSegmentList(0);
@@ -329,7 +328,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
 
  /* Print the start message */
 
- printf_first("Merging Segments: Segments=0 Super-Segments=0 Merged=0 Added=0");
+ printf_first("Merging Segments: Segments=0 Super=0 Merged=0 Added=0");
 
  /* Map into memory / open the files */
 
@@ -358,7 +357,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
           segmentx->node2   ==supersegmentx->node2 &&
           segmentx->distance==supersegmentx->distance)
          {
-          m++;
+          merged++;
           j++;
           /* mark as super-segment and normal segment */
           super=1;
@@ -372,7 +371,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
          {
           /* mark as super-segment */
           AppendSegment(mergedsegmentsx,supersegmentx->way,supersegmentx->node1,supersegmentx->node2,supersegmentx->distance|SEGMENT_SUPER);
-          a++;
+          added++;
           j++;
          }
        else
@@ -388,7 +387,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
        AppendSegment(mergedsegmentsx,segmentx->way,segmentx->node1,segmentx->node2,segmentx->distance|SEGMENT_NORMAL);
 
     if(!((i+1)%10000))
-       printf_middle("Merging Segments: Segments=%d Super-Segments=%d Merged=%d Added=%d",i+1,j,m,a);
+       printf_middle("Merging Segments: Segments=%d Super=%d Merged=%d Added=%d",i+1,j,merged,added);
    }
 
  /* Unmap from memory / close the files */
@@ -405,7 +404,7 @@ SegmentsX *MergeSuperSegments(SegmentsX *segmentsx,SegmentsX *supersegmentsx)
 
  /* Print the final message */
 
- printf_last("Merged Segments: Segments=%d Super-Segments=%d Merged=%d Added=%d",segmentsx->number,supersegmentsx->number,m,a);
+ printf_last("Merged Segments: Segments=%d Super=%d Merged=%d Added=%d",segmentsx->number,supersegmentsx->number,merged,added);
 
  return(mergedsegmentsx);
 }
