@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/CVS/routino/src/typesx.h,v 1.6 2010-12-12 16:35:55 amb Exp $
-
  Type definitions for eXtended types.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -29,6 +27,28 @@
 #include <stdint.h>
 
 
+/* Constants and macros for handling them */
+
+/*+ An undefined node ID. +*/
+#define NO_NODE_ID     (~(node_t)0)
+
+/*+ An undefined way ID. +*/
+#define NO_WAY_ID      (~(way_t)0)
+
+/*+ An undefined relation ID. +*/
+#define NO_RELATION_ID (~(relation_t)0)
+
+/*+ The maximum number of segments per node (used to size temporary storage). +*/
+#define MAX_SEG_PER_NODE 32
+
+
+/* Macro functions */
+
+#define ClearBit(xx,yy)    (xx)[(yy)/8]&=~(1<<((yy)%8))
+#define SetBit(xx,yy)      (xx)[(yy)/8]|= (1<<((yy)%8))
+#define IsBitSet(xx,yy)   ((xx)[(yy)/8]&  (1<<((yy)%8)))
+
+
 /* Simple Types */
 
 /*+ A node identifier - must be at least as large as index_t. +*/
